'
' FILE          Tester.vb
'
' DESCRIPTION   Run client instance(s) of tests whose commands are
'               stored in a file.
'


Option Compare Binary

Class Tester

    Public Instance As Integer                ' Instance number
    Public ConnectString As String            ' Connection string to the DB
    Public WaitInterval As Integer            ' Pause (in ms) between commands
    Public CommandFile As String              ' Name of the file containing scripts
    Public LogFile As String                  ' Name of the file for logging results
    Public Worker As Thread                   ' Thread associated to the main method

    ' Variables to manage the command scripts
    Private CommandFileContent() As String
    Private CommandFileLen As Integer
    Private CurrentCommand As Integer

    Private Shared myMutex As New Mutex     ' Mutex to protect concurrent access to file system and UI controls

    Private AnyLogged As Boolean = False    ' Whether we logged yet

    ' Reference to UI object for UI updating
    Public DBconnection As TextBox
    Public RunningInstances As TextBox
    Public DoneCommands As TextBox

    ''''''''''''''''''''''''''''''''''''''
    ' Subroutine:  StartTester
    ' Parameters:  None
    ' Description: Clears the log file, loads the command file, and runs tests with logging
    ''''''''''''''''''''''''''''''''''''''
    Public Sub StartTester()
        Dim cnn As SqlConnection
        Dim cmd As SqlCommand

        Try
            ClearLog()

            myMutex.WaitOne()
            RunningInstances.Text = Convert.ToString(Convert.ToInt32(RunningInstances.Text) + 1)
            Application.DoEvents()
            myMutex.ReleaseMutex()

            LoadCommandFile()             ' Load the command scripts in memory

            Dim TimeStart As DateTime
            Dim TimeDuration As TimeSpan
            CurrentCommand = 0
            cnn = New SqlConnection(ConnectString)
            cmd = cnn.CreateCommand()
            cmd.CommandType = CommandType.Text

            ' Make sure we are connected (if not, exception occurs)
            cnn.Open()
            cnn.Close()
            DBconnection.Text = "True"

            ' Loop on the command scripts and execute them
            Do While CurrentCommand < CommandFileLen
                cmd.CommandText = CommandFileContent(CurrentCommand)
                cnn.Open()
                TimeStart = DateTime.Now                          ' Start timing
                cmd.ExecuteNonQuery()
                TimeDuration = DateTime.Now.Subtract(TimeStart)   ' End timing
                cnn.Close()

                myMutex.WaitOne()
                Log(DateTime.Now.ToString() & "," & Instance & "," & CurrentCommand + 1 & "," & TimeDuration.TotalMilliseconds)
                DoneCommands.Text = Convert.ToString(Convert.ToInt32(DoneCommands.Text) + 1)  ' Increment the done command counter on UI
                Application.DoEvents()
                myMutex.ReleaseMutex()

                CurrentCommand += 1
                If CurrentCommand < CommandFileLen Then
                    Thread.Sleep(WaitInterval)    ' Pause between commands
                End If
            Loop

        Catch exc As Exception
            'Only show selected errors
            If [String].Compare(exc.Message, "access denied") <> 0 Then
                MessageBox.Show(exc.Message, _
                        "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End If
        Finally
            SyncLock RunningInstances     ' Decrement the instances counter on UI
                RunningInstances.Text = Convert.ToString(Convert.ToInt32(RunningInstances.Text) - 1)
            End SyncLock
            Application.DoEvents()
        End Try
    End Sub

    Private Sub LoadCommandFile()
        CommandFileLen = ScanCommandFile(CommandFile, False)    ' Retrieve the number of scripts in the command file
        ReDim CommandFileContent(CommandFileLen - 1)
        ScanCommandFile(CommandFile, True)                      ' Load the scripts in a memory array
    End Sub

    Private Function ScanCommandFile(ByVal Filename As String, ByVal Read As Boolean) As Integer
        Dim c As Integer = 0                    ' Counter of non-comment lines
        Dim f As StreamReader = File.OpenText(Filename)
        Dim s As String = f.ReadLine()
        Do Until s Is Nothing
            If Not s.StartsWith("--") Then      ' Comment lines start with "--" and are ignored
                If Read Then
                    CommandFileContent(c) = s   ' Load in memory each single scripts, if requested with Read=true (else, only count non-comment lines)
                End If
                c += 1
            End If
            s = f.ReadLine()
        Loop
        f.Close()
        Return c                                ' Return the number of non-comment lines contained in the command file
    End Function

    Private Sub ClearLog()
        Try     'Write a row to the CSV output file for column headings
            Dim w As StreamWriter = File.CreateText(LogFile)
            w.WriteLine("{0}", "DateTime,Instance,Command,Duration")
            w.Flush()
            w.Close()
        Catch   ' Any error is ignored
        End Try
    End Sub
    Private Sub Log(ByVal LogMessage As String)
        Try     ' Append a row to the CSV output file
            Dim w As StreamWriter = File.AppendText(LogFile)
            w.WriteLine("{0}", LogMessage)
            w.Flush()
            w.Close()
        Catch   ' Any error is ignored
        End Try
    End Sub

End Class
